@@ -1,52 +1,84 @@
-/*   FILE: fmt_ptrn.c --
- * AUTHOR: W. Michael Petullo <new@flyn.org>
- *   DATE: 08 January 2000
- *
- * Copyright (c) 1999 W. Michael Petullo <new@flyn.org>
- * All rights reserved.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+/*=============================================================================
+fmt_ptrn.c
+  Copyright (C) W. Michael Putello <new@flyn.org>, 1999 - 2000
+  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006
 
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as
+  published by the Free Software Foundation; either version 2.1 of
+  the License, or (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+  Lesser General Public License for more details.
+
+  You should have received a copy of the GNU Lesser General Public
+  License along with this program; if not, write to:
+  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+  Boston, MA  02110-1301  USA
+
+  -- For details, see the file named "LICENSE.LGPL2"
+=============================================================================*/
 #include <assert.h>
-#include <new/fmt_ptrn.h>
-#include <new/sizes.h>
-#include <new/common.h>
-#include <new/template.h>
-#include <new/modifiers.h>
-#include <new/buffer.h>
-#include <config.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+#include <glib.h>
 #include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <zlib.h>
+#include "buffer.h"
+#include "compiler.h"
+#include "modifiers.h"
+#include "fmt_ptrn.h"
+#include "pair.h"
+
+// Definitions
+#define KEY_LEN         80
+#define PARSE_ERR_LEN   BUFSIZ
+#define STACK_MAX_ITEMS 10
+
+struct mystack {
+    struct modifier data[STACK_MAX_ITEMS];
+    int size;
+};
+
+// Functions
+static void     _apply_modifiers(struct fmt_ptrn *, struct buffer *, struct mystack *);
+static gint     _cmp(gconstpointer, gconstpointer);
+static gboolean _copy_fillers(gpointer, gpointer, gpointer);
+static void     _eat_alternate(struct fmt_ptrn *, const char **);
+static gboolean _fill_it(struct fmt_ptrn *, const char *);
+static int      _fmt_ptrn_copy_fillers(struct fmt_ptrn *, struct fmt_ptrn *);
+static gboolean _fmt_ptrn_t_valid(const struct fmt_ptrn *);
+static gboolean _free_tree_node(gpointer, gpointer, gpointer);
+static void     _handle_fmt_str(struct fmt_ptrn *, const char **);
+static gboolean _is_literal(struct fmt_ptrn *, const char *);
+static gboolean _lookup(const struct fmt_ptrn *, const char *, struct buffer *);
+static char *   _matching_paren(const char *);
+static inline gboolean _modifier_t_valid(const struct modifier *);
+static void     _read_alternate(struct fmt_ptrn *, const char **, struct buffer *);
+static void     _read_key(struct fmt_ptrn *, char *, const char **);
+static void     _read_literal(struct fmt_ptrn *, char *, struct buffer *);
+static gboolean _read_modifier(struct fmt_ptrn *, const char **, struct mystack *);
+static void     _read_modifier_arg(struct fmt_ptrn *, const char **, struct modifier *);
+static void     _read_modifiers(struct fmt_ptrn *, const char **, struct mystack *);
+static int      _stack_contains(const struct mystack, const char *);
+static inline void _stack_init(struct mystack *);
+static gboolean _stack_pop(struct mystack *, struct modifier *);
+static gboolean _stack_push(struct fmt_ptrn *, struct mystack *, const struct modifier *);
+static inline gboolean _stack_t_valid(const struct mystack *);
 
 /* ============================ _fmt_ptrn_t_valid () ======================= */
-static gboolean _fmt_ptrn_t_valid(const fmt_ptrn_t * x)
-{
+static gboolean _fmt_ptrn_t_valid(const struct fmt_ptrn *x) {
 	if (x == NULL)
 		return FALSE;
 	/* FIXME */
 	/* gzFile template_fp; */
 	/* char template_path[PATH_MAX + 1]; */
 	/* long line_num; */
-	if (buffer_t_valid(&x->raw_buf) == FALSE)
-		return FALSE;
-	if (buffer_t_valid(&x->filled_buf) == FALSE)
-		return FALSE;
-	if (buffer_t_valid(&x->lookup_buf) == FALSE)
+        if(!buffer_t_valid(&x->raw_buf) || !buffer_t_valid(&x->filled_buf) ||
+         !buffer_t_valid(&x->lookup_buf))
 		return FALSE;
 	/* char errmsg[BUFSIZ + 1]; */
 	/* GQueue *parse_errmsg; */
@@ -55,24 +87,22 @@ static gboolean _fmt_ptrn_t_valid(const fmt_ptrn_t * x)
 }
 
 /* ============================ fmt_ptrn_parse_err () ====================== */
-gboolean fmt_ptrn_parse_err(const fmt_ptrn_t * x)
-{
+gboolean fmt_ptrn_parse_err(const struct fmt_ptrn *x) {
 	assert(_fmt_ptrn_t_valid(x));
 	return !g_queue_is_empty(x->parse_errmsg);
 }
 
 /* ============================ fmt_ptrn_parse_strerror () ================= */
-char *fmt_ptrn_parse_strerror(fmt_ptrn_t * x)
-{
+char *fmt_ptrn_parse_strerror(struct fmt_ptrn *x) {
 	char *errmsg;
 	char *fnval;
 
 	assert(_fmt_ptrn_t_valid(x));
 
-	if (!(errmsg = g_queue_pop_tail(x->parse_errmsg)))
-		fnval = g_strdup("no error");
+	if((errmsg = g_queue_pop_tail(x->parse_errmsg)) == NULL)
+		fnval = g_strdup("no error"); // leak
 	else
-		fnval = errmsg;
+		fnval = errmsg; // g_queue_pop_tail: possible leak
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(fnval != NULL);
@@ -80,33 +110,15 @@ char *fmt_ptrn_parse_strerror(fmt_ptrn_t * x)
 	return fnval;
 }
 
-/* ============================ fmt_ptrn_parse_perror () =================== */
-void fmt_ptrn_parse_perror(fmt_ptrn_t * x, const char *msg)
-{
-	char *errmsg;
-
-	assert(_fmt_ptrn_t_valid(x));
-
-	errmsg = fmt_ptrn_parse_strerror(x);
-	if (msg)
-		fprintf(stderr, "%s: %s\n", msg, errmsg);
-	else
-		fprintf(stderr, "%s\n", errmsg);
-	g_free(errmsg);
-
-	assert(_fmt_ptrn_t_valid(x));
-}
-
 /* ============================ enqueue_parse_errmsg () =================== */
-void enqueue_parse_errmsg(fmt_ptrn_t * x, const char *msg, ...)
-{
+void enqueue_parse_errmsg(struct fmt_ptrn *x, const char *msg, ...) {
+	va_list args;
 	char *err;
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(msg != NULL);
 
-	err = (char *) g_new0(char, PARSE_ERR_LEN + 1);
-	va_list args;
+	err = g_new0(char, PARSE_ERR_LEN + 1);
 	va_start(args, msg);
 	vsnprintf(err, PARSE_ERR_LEN, msg, args);
 	va_end(args);
@@ -115,50 +127,39 @@ void enqueue_parse_errmsg(fmt_ptrn_t * x, const char *msg, ...)
 	assert(_fmt_ptrn_t_valid(x));
 }
 
-/* ============================ stack_t ==================================== */
-typedef struct stack_t {
-	modifier_t data[STACK_MAX_ITEMS];
-	int size;
-} stack_t;
-
 /* ============================ _modifier_t_valid () ======================= */
-static gboolean _modifier_t_valid(const modifier_t * m)
-{
+static inline gboolean _modifier_t_valid(const struct modifier *m) {
 	/* FIXME */
 	return TRUE;
 }
 
 /* ============================ _stack_t_valid () ========================== */
-static gboolean _stack_t_valid(const stack_t * s)
-{
-	if (s == NULL && s->size != 0)
-		return FALSE;
-	return TRUE;
+static inline gboolean _stack_t_valid(const struct mystack *s) {
+    return (s == NULL && s->size != 0) ? FALSE : TRUE;
 }
 
 /* ============================ _stack_init () ============================= */
-static void _stack_init(stack_t * s)
-{
+static inline void _stack_init(struct mystack *s) {
 	s->size = 0;
 	assert(_stack_t_valid(s));
 }
 
 /* ============================ _stack_push () ============================= */
-static gboolean _stack_push(fmt_ptrn_t * x, stack_t * s,
-			    const modifier_t data)
+static gboolean _stack_push(struct fmt_ptrn *x, struct mystack *s,
+ const struct modifier *data)
 {
 	gboolean fnval = FALSE;
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(_stack_t_valid(s));
-	assert(_modifier_t_valid(&data));
+	assert(_modifier_t_valid(data));
 	if (s->size == STACK_MAX_ITEMS) {
 		enqueue_parse_errmsg(x, "%s: %ld: more than %d modifiers",
 				     x->template_path, x->line_num,
 				     STACK_MAX_ITEMS);
 		fnval = FALSE;
 	} else {
-		s->data[s->size++] = data;
+		s->data[s->size++] = *data;
 		fnval = TRUE;
 	}
 
@@ -168,12 +169,11 @@ static gboolean _stack_push(fmt_ptrn_t * x, stack_t * s,
 }
 
 /* ============================ _stack_pop () ============================== */
-static gboolean _stack_pop(stack_t * s, modifier_t * data)
-{
+static gboolean _stack_pop(struct mystack *s, struct modifier *data) {
 	gboolean fnval = FALSE;
 	assert(_stack_t_valid(s));
 	assert(_modifier_t_valid(data));
-	if (!s->size)
+	if(s->size == 0)
 		fnval = FALSE;
 	else {
 		*data = s->data[--s->size];
@@ -186,7 +186,7 @@ static gboolean _stack_pop(stack_t * s, modifier_t * data)
 }
 
 /* ============================ _stack_contains () ========================= */
-static int _stack_contains(const stack_t s, const char *n)
+static int _stack_contains(const struct mystack s, const char *n)
 {
 	int i;
 	assert(_stack_t_valid(&s));
@@ -196,55 +196,43 @@ static int _stack_contains(const stack_t s, const char *n)
 	return 0;
 }
 
-/* ============================ fmt_ptrn_update_kv_p () ==================== */
-void fmt_ptrn_update_kv_p(fmt_ptrn_t * x, const pair_t * p)
-{
-	assert(_fmt_ptrn_t_valid(x));
-	assert(pair_t_valid(p));
-	/* FIXME: this strdups here but other fn requires malloced strs! */
-	g_tree_insert(x->fillers, strdup(p->key), strdup(p->val));
-	assert(_fmt_ptrn_t_valid(x));
-}
-
 /* ============================ fmt_ptrn_update_kv () ====================== */
-void fmt_ptrn_update_kv(fmt_ptrn_t * x, const char *key, const char *val)
-{
+void fmt_ptrn_update_kv(struct fmt_ptrn *x, const char *key, const char *val) {
 	assert(_fmt_ptrn_t_valid(x));
 	assert(key != NULL);
 	assert(val != NULL);
 	/* FIXME: getting rid of the const is silly, but I didn't write g_tree_insert */
-	g_tree_insert(x->fillers, (char *) key, (char *) val);
+        // string not duplicated but freed? see above
+	g_tree_insert(x->fillers, const_cast(char *, key), const_cast(char *, val));
 	assert(_fmt_ptrn_t_valid(x));
 }
 
 /* ============================ _matching_paren () ========================= */
-static char *_matching_paren(char *str)
-/* feed it a pointer just after a '(' and it will return a pointer to the
- * matching ')'
- */
-{
+static char *_matching_paren(const char *str) {
+    /* Feed it a pointer just after a '(' and it will return a pointer to the
+    matching ')'. Note that it must return CHAR *, not CONST CHAR *, just as
+    strchr() does for example. */
 	int count = 1;
 	assert(str != NULL);
-	while (*str) {
+	while(*str != '\0') {
 		if (*str == '(')
 			count++;
 		else if (*str == ')')
 			count--;
 		if (count == 0)
-			return str;
+			return const_cast(char *, str);
 		str++;
 	}
 	return NULL;
 }
 
 /* ============================ _copy_fillers () =========================== */
-gboolean _copy_fillers(gpointer key, gpointer val, gpointer data)
-{
+static gboolean _copy_fillers(gpointer key, gpointer val, gpointer data) {
 	assert(key != NULL);
 	assert(val != NULL);
 	assert(_fmt_ptrn_t_valid(data));
 
-	g_tree_insert(((fmt_ptrn_t *) data)->fillers,
+	g_tree_insert(static_cast(struct fmt_ptrn *, data)->fillers,
 		      strdup(key), strdup(val));
 
 	assert(_fmt_ptrn_t_valid(data));
@@ -253,9 +241,8 @@ gboolean _copy_fillers(gpointer key, gpointer val, gpointer data)
 }
 
 /* ============================ _fmt_ptrn_copy_fillers () ================== */
-int _fmt_ptrn_copy_fillers(fmt_ptrn_t * x, fmt_ptrn_t * y)
+static int _fmt_ptrn_copy_fillers(struct fmt_ptrn *x, struct fmt_ptrn *y) {
 /* Copies fillers from one fmt_ptrn to another. */
-{
 	assert(_fmt_ptrn_t_valid(x));
 	assert(_fmt_ptrn_t_valid(y));
 
@@ -269,20 +256,21 @@ int _fmt_ptrn_copy_fillers(fmt_ptrn_t * x, fmt_ptrn_t * y)
 }
 
 /* ============================ _read_alternate () ========================= */
-static void _read_alternate(fmt_ptrn_t * x, char **p, buffer_t * buf)
+static void _read_alternate(struct fmt_ptrn *x, const char **p,
+ struct buffer *buf)
 {
 	char *alt_end;
 	assert(_fmt_ptrn_t_valid(x));
 	assert(p != NULL);
 	assert(*p != NULL);
 	assert(buffer_t_valid(buf));
-	if (!**p)		/* Already queued error, hopefully. */
+	if(**p == '\0')		/* Already queued error, hopefully. */
 		return;
 	if (**p == ':') {
-		(*p)++;
-		if ((alt_end = _matching_paren(*p))) {
+		++*p;
+		if((alt_end = _matching_paren(*p)) != NULL) {
 			/* FIXME: clean up? */
-			fmt_ptrn_t y;
+			struct fmt_ptrn y;
 			char *alt, *filled_alt;
 			alt = g_strndup(*p, (alt_end - *p));
 			fmt_ptrn_init(&y);
@@ -315,18 +303,17 @@ static void _read_alternate(fmt_ptrn_t * x, char **p, buffer_t * buf)
 }
 
 /* ============================ _eat_alternate () ========================== */
-static void _eat_alternate(fmt_ptrn_t * x, char **pattern)
-{
+static void _eat_alternate(struct fmt_ptrn *x, const char **pattern) {
 	char *alt_end;
 	assert(_fmt_ptrn_t_valid(x));
 	assert(pattern != NULL);
 	assert(*pattern != NULL);
-	if (!**pattern || **pattern != ':')
+	if(**pattern == '\0' || **pattern != ':')
 		/* No alternate provided to eat. */
 		return;
-	if ((alt_end = _matching_paren(*pattern)))
+	if((alt_end = _matching_paren(*pattern)) != NULL)
 		*pattern += (alt_end - *pattern);
-	if (!**pattern)
+	if(**pattern == '\0')
 		enqueue_parse_errmsg(x, "%s: %ld: end of input",
 				     x->template_path, x->line_num);
 	assert(_fmt_ptrn_t_valid(x));
@@ -335,8 +322,8 @@ static void _eat_alternate(fmt_ptrn_t * x, char **pattern)
 }
 
 /* ============================ _read_modifier_arg () ====================== */
-static void _read_modifier_arg(fmt_ptrn_t * x,
-			       char **pattern, modifier_t * i)
+static void _read_modifier_arg(struct fmt_ptrn *x, const char **pattern,
+ struct modifier *i)
 {
 	size_t arg_len;
 	char *end_quote, *end_paren;
@@ -348,7 +335,7 @@ static void _read_modifier_arg(fmt_ptrn_t * x,
 
 	end_quote = strchr(*pattern, '"');
 	end_paren = strchr(*pattern, ')');
-	if (!end_quote || (end_paren && end_quote > end_paren))
+	if(end_quote == NULL || (end_paren != NULL && end_quote > end_paren))
 		enqueue_parse_errmsg(x, "%s: %ld: no end quote",
 				     x->template_path, x->line_num);
 	else {
@@ -376,11 +363,11 @@ static void _read_modifier_arg(fmt_ptrn_t * x,
 }
 
 /* ============================ _read_modifier () ========================== */
-static gboolean _read_modifier(fmt_ptrn_t * x, char **ptrn,
-			       stack_t * modifier)
+static gboolean _read_modifier(struct fmt_ptrn *x, const char **ptrn,
+ struct mystack *modifier)
 {
 	int i = 0;
-	modifier_t m;
+	struct modifier m;
 	gboolean fnval = FALSE;
 
 	assert(_fmt_ptrn_t_valid(x));
@@ -388,8 +375,8 @@ static gboolean _read_modifier(fmt_ptrn_t * x, char **ptrn,
 	assert(*ptrn != NULL);
 	assert(_stack_t_valid(modifier));
 
-	while (mod_fn[i].id) {
-		if (!strncmp(mod_fn[i].id, *ptrn, strlen(mod_fn[i].id))) {
+	while(mod_fn[i].id != NULL) {
+		if(strncmp(mod_fn[i].id, *ptrn, strlen(mod_fn[i].id)) == 0) {
 			*ptrn +=
 			    strlen(mod_fn[i].id) +
 			    (mod_fn[i].has_arg ? 0 : 1);
@@ -397,8 +384,8 @@ static gboolean _read_modifier(fmt_ptrn_t * x, char **ptrn,
 			if (mod_fn[i].has_arg)
 				_read_modifier_arg(x, ptrn, &m);
 			else
-				*m.arg = 0x00;
-			_stack_push(x, modifier, m);
+				*m.arg = '\0';
+			_stack_push(x, modifier, &m);
 			fnval = TRUE;
 			break;
 		}
@@ -414,17 +401,15 @@ static gboolean _read_modifier(fmt_ptrn_t * x, char **ptrn,
 }
 
 /* ============================ _read_modifiers () ========================= */
-static void _read_modifiers(fmt_ptrn_t * x,
-			    char **ptrn, stack_t * modifier)
+static void _read_modifiers(struct fmt_ptrn *x, const char **ptrn,
+ struct mystack *modifier)
 {
 	assert(_fmt_ptrn_t_valid(x));
 	assert(ptrn != NULL);
 	assert(*ptrn != NULL);
 	assert(_stack_t_valid(modifier));
 
-	while (_read_modifier(x, ptrn, modifier)) {
-		/* NOOP. */
-	}
+	while(_read_modifier(x, ptrn, modifier)) /* noop */;
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(ptrn != NULL);
@@ -435,8 +420,7 @@ static void _read_modifiers(fmt_ptrn_t * x,
 }
 
 /* ============================ _read_key () =============================== */
-static void _read_key(fmt_ptrn_t * x, char *key, char **p)
-{
+static void _read_key(struct fmt_ptrn *x, char *key, const char **p) {
 	int i;
 
 	assert(_fmt_ptrn_t_valid(x));
@@ -444,17 +428,17 @@ static void _read_key(fmt_ptrn_t * x, char *key, char **p)
 	assert(p != NULL);
 	assert(*p != NULL);
 
-	*key = 0x00;
-	for (i = 0; i < KEY_LEN && **p && !strchr(":)", **p); i++)
+	*key = '\0';
+	for(i = 0; i < KEY_LEN && **p != '\0' && strchr(":)", **p) == NULL; i++)
 		strncat(key, (*p)++, 1);
-	if (**p && !strchr(":)", **p)) {
+	if(**p != '\0' && strchr(":)", **p) == NULL) {
 		/* Uh oh, key is too many characters, eat the rest. */
-		while (**p && **p != ':' && **p != ')')
-			(*p)++;
+		while(**p != '\0' && **p != ':' && **p != ')')
+			++*p;
 		enqueue_parse_errmsg(x, "%s: %ld: key too long",
 				     x->template_path, x->line_num);
 	}
-	if (!**p)
+	if(**p == '\0')
 		enqueue_parse_errmsg(x, "%s: %ld: end of input",
 				     x->template_path, x->line_num);
 
@@ -464,26 +448,20 @@ static void _read_key(fmt_ptrn_t * x, char *key, char **p)
 }
 
 /* ============================ _apply_modifiers () ======================== */
-static void _apply_modifiers(fmt_ptrn_t * x,
-			     buffer_t * str, stack_t * modifier)
+static void _apply_modifiers(struct fmt_ptrn *x, struct buffer *str,
+ struct mystack *modifier)
 {
-	modifier_t m;
+	struct modifier m;
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(buffer_t_valid(str));
 	assert(_stack_t_valid(modifier));
 
 	if (buffer_len(str) > 0)	/* error should have been queued elsewhere */
-		while (_stack_pop(modifier, &m))
-			if (m.fn.fn)
-				if (!m.fn.fn(str, x, m.arg))
-					enqueue_parse_errmsg(x,
-							     "%s: %ld: error applying %s modifier to %s",
-							     x->
-							     template_path,
-							     x->line_num,
-							     m.fn.id,
-							     str->data);
+            while(_stack_pop(modifier, &m))
+                if(m.fn.fn != NULL && !m.fn.fn(str, x, m.arg))
+                    enqueue_parse_errmsg(x, "%s: %ld: error applying %s modifier to %s",
+                      x->template_path, x->line_num, m.fn.id, str->data);
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(buffer_t_valid(str));
@@ -491,7 +469,8 @@ static void _apply_modifiers(fmt_ptrn_t * x,
 }
 
 /* ============================ _lookup () ================================= */
-gboolean _lookup(const fmt_ptrn_t * x, const char *key, buffer_t * value)
+static gboolean _lookup(const struct fmt_ptrn *x, const char *key,
+ struct buffer *value)
 {
 	char *tmp;
 	gboolean fnval = FALSE;
@@ -501,12 +480,12 @@ gboolean _lookup(const fmt_ptrn_t * x, const char *key, buffer_t * value)
 	assert(buffer_t_valid(value));
 
 	tmp = g_tree_lookup(x->fillers, key);
-	if (tmp) {
+	if(tmp != NULL) {
 		realloc_n_cpy(value, tmp);
 		fnval = TRUE;
 	} else {
 		if (buffer_len(value) > 0) /* error should have been queue elsewhere */
-			*value->data = 0x00;	/* Otherwise _read_alternate will append onto garbage. */
+			*value->data = '\0';	/* Otherwise _read_alternate will append onto garbage. */
 		fnval = FALSE;
 	}
 
@@ -516,15 +495,14 @@ gboolean _lookup(const fmt_ptrn_t * x, const char *key, buffer_t * value)
 }
 
 /* ============================ _is_literal () ============================= */
-static gboolean _is_literal(fmt_ptrn_t * x, char *str)
-{
+static gboolean _is_literal(struct fmt_ptrn *x, const char *str) {
 	gboolean fnval = FALSE;
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(str != NULL);
 
 	if (*str == '"') {
-		if (strchr(str + 1, '"'))
+		if(strchr(str + 1, '"') != NULL)
 			enqueue_parse_errmsg(x, "%s: %ld: no end quote",
 					     x->template_path,
 					     x->line_num);
@@ -538,15 +516,14 @@ static gboolean _is_literal(fmt_ptrn_t * x, char *str)
 
 /* ============================ _read_literal () =========================== */
 /* FIXME: is this right?  it does not seem to look for closing '"' */
-static void _read_literal(fmt_ptrn_t *x, char *str, buffer_t * buf)
-{
+static void _read_literal(struct fmt_ptrn *x, char *str, struct buffer *buf) {
 	assert(_fmt_ptrn_t_valid(x));
 	assert(str != NULL);
 	assert(buffer_t_valid(buf));
 
 	str++;
-	str[strlen(str) - 1] = 0x00;
-	if (strlen(str))
+	str[strlen(str) - 1] = '\0';
+	if(strlen(str) > 0)
 		realloc_n_cpy(buf, str);
 	else
 		enqueue_parse_errmsg(x, "%s: %ld: no literal found in quotes",
@@ -558,10 +535,9 @@ static void _read_literal(fmt_ptrn_t *x, char *str, buffer_t * buf)
 }
 
 /* ============================ _handle_fmt_str () ========================= */
-static void _handle_fmt_str(fmt_ptrn_t * x, char **p)
-{
+static void _handle_fmt_str(struct fmt_ptrn *x, const char **p) {
 	/* format string -> %(<modifier_0> ... <modifier_n> <key>:<alt>) */
-	stack_t modifier;
+	struct mystack modifier;
 	char key[KEY_LEN + 1];
 
 	assert(_fmt_ptrn_t_valid(x));
@@ -587,8 +563,8 @@ static void _handle_fmt_str(fmt_ptrn_t * x, char **p)
 			/* error should have been queued elsewhere */
 			realloc_n_cat(&x->filled_buf, x->lookup_buf.data);
 	}
-	if (**p)
-		(*p)++;		/* Skip ')'. */
+	if(**p != '\0')
+		++*p; /* Skip ')'. */
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(p != NULL);
@@ -596,23 +572,23 @@ static void _handle_fmt_str(fmt_ptrn_t * x, char **p)
 }
 
 /* ============================ _fill_it () ================================ */
-static gboolean _fill_it(fmt_ptrn_t * x, const char *p)
-{
-	char *pattern, *orig_ptr;
+static gboolean _fill_it(struct fmt_ptrn *x, const char *p) {
+	const char *pattern;
+        char *orig_ptr;
 	gboolean fnval = TRUE;
 
 	assert(_fmt_ptrn_t_valid(x));
 	assert(p != NULL);
 
-	pattern = orig_ptr = g_strdup((char *) p); /* stupid cast */
-	while (*pattern != 0x00) {
+	pattern = orig_ptr = g_strdup(p);
+	while(*pattern != '\0') {
 		if (*pattern == '%' && *(pattern + 1) == '%') {
 			/* Handle %%(...), which should be filled as %(...). */
 			realloc_n_ncat(&x->filled_buf, pattern, 1);
 			pattern += 2;
-		} else if (*pattern == '%' && *(pattern + 1) == '(')
+		} else if(*pattern == '%' && *(pattern + 1) == '(') {
 			_handle_fmt_str(x, &pattern);
-		else {
+		} else {
 			if (*pattern == '\n')
 				x->line_num++;
 			realloc_n_ncat(&x->filled_buf, pattern++, 1);
@@ -626,8 +602,7 @@ static gboolean _fill_it(fmt_ptrn_t * x, const char *p)
 }
 
 /* ============================ fmt_ptrn_filled () ========================= */
-char *fmt_ptrn_filled(fmt_ptrn_t * x, const char *p)
-{
+char *fmt_ptrn_filled(struct fmt_ptrn *x, const char *p) {
 	char *fnval = NULL;
 
 	assert(_fmt_ptrn_t_valid(x));
@@ -636,7 +611,7 @@ char *fmt_ptrn_filled(fmt_ptrn_t * x, const char *p)
 	buffer_clear(&x->filled_buf);
 	if (!_fill_it(x, p))
 		return NULL;
-	if (buffer_len(&x->filled_buf))	
+	if(buffer_len(&x->filled_buf) > 0)
 		/* FIXME: what if len == 0? protected by assert, but... */
 		fnval = g_strdup(x->filled_buf.data);
 
@@ -647,32 +622,30 @@ char *fmt_ptrn_filled(fmt_ptrn_t * x, const char *p)
 }
 
 /* ============================ _cmp () ==================================== */
-gint _cmp(gconstpointer a, gconstpointer b)
-{
+static gint _cmp(gconstpointer a, gconstpointer b) {
 	/* FIXME: why is a and/or b sometimes NULL? */
-	if (!a && !b)
+	if(a == NULL && b == NULL)
 		return 0;
-	if (!a)
+	if(a == NULL)
 		return -1;
-	if (!b)
+	if(b == NULL)
 		return 1;
 	return strcmp(a, b);
 }
 
 /* ============================ fmt_ptrn_init () =========================== */
-int fmt_ptrn_init(fmt_ptrn_t * x)
+int fmt_ptrn_init(struct fmt_ptrn *x) {
 /* Alternative to open; does everything but open the file.  This 
  * should be used when filling strings instead of files.
  */
-{
 	g_strlcpy(x->errmsg, "no error", sizeof(x->errmsg));
 	x->parse_errmsg = g_queue_new();
 	x->fillers = g_tree_new(_cmp);
 	x->template_fp = NULL;
 	x->line_num = 1;
-	x->raw_buf = buffer_init();
-	x->filled_buf = buffer_init();
-	x->lookup_buf = buffer_init();
+        buffer_init(&x->raw_buf);
+        buffer_init(&x->filled_buf);
+        buffer_init(&x->lookup_buf);
 	g_strlcpy(x->template_path, "string", sizeof(x->template_path));
 	
 	assert(_fmt_ptrn_t_valid(x));
@@ -681,15 +654,14 @@ int fmt_ptrn_init(fmt_ptrn_t * x)
 }
 
 /* ============================ fmt_ptrn_open () =========================== */
-gboolean fmt_ptrn_open(const char *path, fmt_ptrn_t * x)
-{
+gboolean fmt_ptrn_open(const char *path, struct fmt_ptrn *x) {
 	gzFile in_file;
 	gboolean fnval = TRUE;
 
 	assert(path != NULL);
 	assert(_fmt_ptrn_t_valid(x));
 
-	if (!(in_file = gzopen(path, "rb"))) {
+	if((in_file = gzopen(path, "rb")) == NULL) {
 		fnval = FALSE;
 		goto _return;
 	} 
@@ -706,8 +678,7 @@ _return:
 }
 
 /* ============================ fmt_ptrn_gets () =========================== */
-char *fmt_ptrn_gets(char *buf, size_t size, fmt_ptrn_t * x)
-{
+char *fmt_ptrn_gets(char *buf, size_t size, struct fmt_ptrn *x) {
 	char *fnval = NULL;
 
 	assert(buf != NULL);
@@ -743,15 +714,13 @@ _return:
 
 /* ============================ _free_tree_node () ========================= */
 /* FIXME: this function should take TWO pointers!!!!!! */
-gboolean _free_tree_node(gpointer key, gpointer val, gpointer data)
-{
+static gboolean _free_tree_node(gpointer key, gpointer val, gpointer data) {
 /* FIXME: this function may not modify tree.  need to write pointers to a list and then destroy that list outside of this function. */
 	return FALSE;
 }
 
 /* ============================ fmt_ptrn_close () ========================== */
-int fmt_ptrn_close(fmt_ptrn_t * x)
-{
+int fmt_ptrn_close(struct fmt_ptrn *x) {
 	gpointer ptr;
 
 	assert(_fmt_ptrn_t_valid(x));
@@ -759,28 +728,14 @@ int fmt_ptrn_close(fmt_ptrn_t * x)
 	while ((ptr = g_queue_pop_head(x->parse_errmsg)) != NULL)
 		g_free(ptr);
 	g_tree_foreach(x->fillers, _free_tree_node, NULL);
-	buffer_destroy(x->raw_buf);
-	buffer_destroy(x->filled_buf);
-	buffer_destroy(x->lookup_buf);
+	buffer_clear(&x->raw_buf);
+	buffer_clear(&x->filled_buf);
+	buffer_clear(&x->lookup_buf);
 	/* x->template_fp == NULL if fmt_ptrn_init was used instead of 
 	 * fmt_ptrn_open.
 	 */
-	return x && x->template_fp ? gzclose(x->template_fp) : 1;
+	return (x != NULL && x->template_fp != NULL) ?
+            gzclose(x->template_fp) : 1;
 }
 
-/* ============================ fmt_ptrn_perror () ========================= */
-void fmt_ptrn_perror(const fmt_ptrn_t * x, const char *msg)
-{
-	assert(_fmt_ptrn_t_valid(x));
-	assert(msg != NULL);
-
-	fprintf(stderr, "%s: %s\n", msg, x->errmsg);
-}
-
-/* ============================ fmt_ptrn_strerror () ======================= */
-const char *fmt_ptrn_strerror(const fmt_ptrn_t * x)
-{
-	assert(_fmt_ptrn_t_valid(x));
-
-	return x->errmsg;
-}
+//=============================================================================
